import toml
import os

def list_channels(directory: str):
    path = os.path.join(directory, "cache.toml")
    try:
        f = open(path, "r", encoding="utf-8")
        data = toml.loads(f.read())
        f.close()
        return data
    except FileNotFoundError:
        init_cache(directory)
        print("No saved channels")
    except:
        print("Error reading cache")

def add_channel(data, channel):
    if data['channels'] is None:
        return channel
    data['channels'].update(channel)
    return data

def remove_channel(data: dict, channel: str):
    if data.get('channels') is not None and data['channels'].get(channel) is not None:
        data['channels'].pop(channel, None)
    return data

def read(directory: str):
    path = os.path.join(directory, "cache.toml")
    try:
        f = open(path, "r", encoding="utf-8")
        data = toml.loads(f.read())
        f.close()
        if data.get("channels") is None:
            data.update({"channels": {}})
        return data
    except FileNotFoundError:
        init_cache(directory)
        return {"channels": {}}

def save(data, directory: str):
    path = os.path.join(directory, "cache.toml")
    try:
        f = open(path, "w", encoding="utf-8")
        f.write(toml.dumps(data))
        f.close()
    except:
        print("Error saving cache")

def init_cache(directory: str):
    os.makedirs(directory, exist_ok=True)
    path = os.path.join(directory, "cache.toml")
    f = open(path, "w", encoding="utf-8")
    f.write(toml.dumps({"channels": {}}))
    f.close()

def export_saved(directory: str):
    path = os.path.join(directory, "cache.toml")
    f = open(path, "r", encoding="utf-8")
    data = toml.loads(f.read())
    f.close()
    result = ""
    for channel in data['channels']:
        result += "\"" + channel + "\" "
    return result

def remove_all(directory: str):
    path = os.path.join(directory, "cache.toml")
    os.remove(path)

def get_channel_name(id: str, data: dict):
    return data['channels'][id]['title']

def get_saved_channels(data: dict):
    result = list()
    for id in data['channels']:
        result.append(data['channels'][id]['title'])
    return result



import youtubeinfo as yi
from datetime import datetime

def convert_channel_info(data): 
    return {'url': data['url'], 'title': data['title'], 'subscribers': data['subscribers']['simpleText'], 'views': data['views']}

def convert_videos_output(data):
    result = ""
    for video in data:
        result += "Title: " + video['title'] + "\nChannel: " + video['channel']['name'] + "\nUpload Date: " + video['uploadDate'] + "\nLink: " + video['link'] + "\n-------------------------------------------\n\n"
    return result

def convert_channel_videos_output(data: list, channel_id: str, limit: int):
    result = "Channel Name: " + data[0]['channel']['name'] + "\nChannel Link: https://youtube.com/channel/" + channel_id + "\n-------------------------------------------\n\n"
    for i in range(limit):
        link = data[i]['link'].split('&')[0]
        upload_date = yi.get_video_upload_date(link)
        result += "Title: " + data[i]['title'] + "\nLink: " + link + "\nUpload Date: " + datetime.strptime(upload_date, "%Y%m%d").strftime("%Y-%m-%d") + "\n-------------------------------------------\n\n"
    return result

def convert_channel_list(data: dict):
    result = ""
    for channel in data['channels']:
        result += "Name: " + data['channels'][channel]['title'] + "\nLink: " + data['channels'][channel]['url'] + "\nChannel ID: " + channel + "\n-------------------------------------------\n\n" 
    return result


import yt_dlp
import os
from datetime import datetime
import youtubeinfo as yi
from pathlib import Path

# video_info {'publishDate', 'url', 'channel', 'title', 'id'}
def download(video_info: dict, video_format: str, directory: str):

    video_info['channel'] = remove_filename_unsopported_chars(video_info["channel"])
    video_info['title'] = remove_filename_unsopported_chars(video_info["title"])

    upload_date = video_info['publishDate'].strftime("%Y_%m_%d")
    path = os.path.join(directory, upload_date)
    path = os.path.join(path, video_info['channel'])

    ydl_opts = {
        "format": video_format,
        "paths": {"home": path},
        "quiet": True,
        "outtmpl": {"default": "PLACEHOLDER.%(ext)s"},
        "no_warnings": True
        # "progress_hooks": {
        #     "status": True,
        #     "filename": True,
        #     "eta": True
        # }
    }
    if not os.path.isdir(directory):
        os.mkdir(directory)
    if not os.path.isdir(os.path.join(directory, upload_date)):
        os.mkdir(os.path.join(directory, upload_date))
    if not os.path.isdir(path):
        os.mkdir(path)
    file_name = video_info['title']
    for file in os.listdir(path):
        if Path(file).stem == file_name:
            return
    print(upload_date + "/" + video_info['channel'] + "/" + video_info['title'] + ": Downloading...")
    try:
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            ydl.download(video_info['url'])
        for file in os.listdir(path):
            if Path(file).stem  == "PLACEHOLDER":
                os.rename(os.path.join(path, file), os.path.join(path, video_info['title'] + os.path.splitext(file)[1]))
    except:
        print("Couldn't download video: " + video_info['url'])
        for file in os.listdir(path):
            if Path(file).stem  == "PLACEHOLDER":
                os.rename(os.path.join(path, file), os.path.join(path, video_info['title'] + os.path.splitext(file)[1]))

def download_channel(id: str, video_format: str, directory: str, max_video_age: int, limit: int):
    videos = yi.find_channel_videos(id, limit)
    for video in videos:
        video['link'] = video['link'].split('&')[0]
        publish_date = datetime.today()
        try: 
            with yt_dlp.YoutubeDL({"quiet": True, "no_warnings": True}) as ydl:
                info_dict = ydl.extract_info(video['link'], download=False)
                if info_dict['is_live']:
                    video_format = "best"
                publish_date = datetime.strptime(info_dict.get("upload_date", datetime.today()), "%Y%m%d")
        except:
            continue
        if max_video_age == 0:
            download({'publishDate': publish_date, 'channel': video['channel']['name'], 'url': video['link'], 'title': video['title'], 'id': video['id']}, video_format, directory)
        else:
            now = datetime.today()
            delta = now - publish_date
            if delta.days <= max_video_age:
                download({'publishDate': publish_date, 'channel': video['channel']['name'], 'url': video['link'], 'title': video['title'], 'id': video['id']}, video_format, directory)
            else:
                return


def remove_filename_unsopported_chars(name: str):
    name = name.replace("\\", "")
    name = name.replace("/", "")
    name = name.replace(":", "")
    name = name.replace("*", "")
    name = name.replace("?", "")
    name = name.replace("\"", "")
    name = name.replace("<", "")
    name = name.replace(">", "")
    name = name.replace("|", "")
    name = name.strip()
    return name


import toml
import os

CONFIG_PATH = os.path.join(os.path.expanduser('~'), ".config/autoyt/config.toml")
CACHE_DIR = os.path.join(os.path.expanduser('~'), ".cache/autoyt")
VIDEOS_PATH = os.path.join(os.path.expanduser('~'), "Videos/autoYT")

def get_download_dir():
    try:
        f = open(os.path.join(CONFIG_PATH))
        data = toml.loads(f.read())
        f.close()
        return data['download_dir']
    except:
        init()
        return VIDEOS_PATH

def get_cache_dir():
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        return data['cache_dir']
    except:
        init()
        return os.path.join(os.path.expanduser('~'), ".cache/autoyt/")
    
def get_all():
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        return data
    except:
        init()
        return {"download_dir": VIDEOS_PATH, "cache_dir": CACHE_DIR, "quality": "ba+bv[height<=?1080]", "max_video_age": 5, "max_downloaded_channel_videos": 5, "delete_videos_age": 5, "channels_check_break": 600}

def get_max_downloaded_channel_videos(channel: str):
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        if data.get(channel) is None:
            return data.get("max_downloaded_channel_videos")
        if data[channel].get("max_downloaded_channel_videos") is None:
            return data.get("max_downloaded_channel_videos")
    except:
        init()
        return 5


def get_download_quality(channel: str):
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        if data.get(channel) is None:
            return data.get("quality")
        if data[channel].get("quality") is None:
            return data.get("quality")
    except:
        init()
        return "ba+bv[height<=?1080]"

def get_max_video_age():
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        return data["max_video_age"]
    except:
        return 5

def get_delete_videos_age():
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        return data["delete_videos_age"]
    except:
        return 5

def get_channels_check_break():
    try:
        f = open(CONFIG_PATH)
        data = toml.loads(f.read())
        f.close()
        return data['channels_check_break']
    except:
        return 600

def init():
    try:
        os.makedirs(os.path.join(os.path.expanduser('~'), ".config/autoyt"), exist_ok=True)
        f = open(CONFIG_PATH, "w")
        f.write(toml.dumps({"download_dir": VIDEOS_PATH, "cache_dir": CACHE_DIR, "quality": "ba+bv[height<=?1080]", "max_video_age": 5, "delete_videos_age": 5, "channels_check_break": 600}))
        f.close()
    except:
        print("Couldn't create config file")

def get_config_dir():
    return os.path.join(os.path.expanduser('~'), ".config/autoyt")


import os
import shutil
from datetime import datetime, timedelta

def remove_old_videos(days: int, directory: str):
    if days == 0:
        return
    max_date = datetime.today() - timedelta(days=days)
    for file in os.listdir(directory):
        d = os.path.join(directory, file)
        if os.path.isdir(d):
            if datetime.strptime(file, "%Y_%m_%d") < max_date:
                shutil.rmtree(d)

def remove_all_videos(directory: str):
    for file in os.listdir(directory):
        d = os.path.join(directory, file)
        shutil.rmtree(d)

def find_videos(name: str, directory: str):
    result = ""
    for file in os.listdir(directory):
        d = os.path.join(directory, file)
        d = os.path.join(d, name) 
        if os.path.isdir(d):
            for vid_name in os.listdir(d):
                result += os.path.join(d, vid_name) + "\n"
    return result

def remove_invalid_videos(saved_channels: list, directory: str):
    for dir in os.listdir(directory):
        d = os.path.join(directory, dir)
        for channel in os.listdir(d):
            if not channel in saved_channels:
                shutil.rmtree(os.path.join(d, channel))
    for dir in os.listdir(directory):
        path = os.path.join(directory, dir)
        if len(os.listdir(path)) == 0:
            os.rmdir(path)

def remove_from_channel(channel: str, directory: str):
    for file in os.listdir(directory):
        d = os.path.join(directory, file)
        for ch in os.listdir(d):
            if ch == channel:
                shutil.rmtree(os.path.join(d, ch)) 

def remove_empty_dirs(directory: str):
    for file in os.listdir(directory):
        d = os.path.join(directory, file)
        if len(os.listdir(d)) == 0:
            shutil.rmtree(d)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="autoyt",
    version="0.1.1",
    author="Two-Six",
    author_email="twopsix@gmail.com",
    url="https://github.com/two-six/autoYT",
    description="Python script for quickly searching and downloading new content out of YouTube.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["autoyt"],
    license="AGPLv3",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    py_modules=["autoyt"],
    package_dir={'':'autoyt/src'},
    install_requires=['yt-dlp', 'ffmpeg']
)


import youtubesearchpython as ysp
import yt_dlp
from datetime import datetime

def find_channel_id(name):
    if is_id(name):
        return name
    videosSearch = ysp.VideosSearch(name, 20)
    channel_id_dict = {}
    for i in videosSearch.result()['result']:
        channel_id = i['channel']['id']
        if channel_id in channel_id_dict:
            channel_id_dict[channel_id] += 1
        else:
            channel_id_dict[channel_id] = 1
    return max(channel_id_dict, key=channel_id_dict.get)

def get_channel_info(id):
    try:
        channel_info = ysp.Channel.get(id)
        return channel_info
    except:
        print("Couldn't get channel information: ", id)
        return {}

def find_channel_videos(channel_id: str, limit: int = 10):
    try:
        playlist = ysp.Playlist(ysp.playlist_from_channel_id(channel_id))
        while playlist.hasMoreVideos and (len(playlist.videos) < limit or limit == 0):
            playlist.getNextVideos()
        if limit == 0:
            return playlist.videos
        return playlist.videos[0:limit]
    except:
        print("Couldn't find channel videos: " + channel_id)
        return {}

def is_id(name):
    try:
        ysp.Playlist(ysp.playlist_from_channel_id(name))
        return True
    except:
        return False

def find_videos(name: str, limit: int = 10):
    return list(map(lambda x: ysp.Video.get(x['link'], get_upload_date=True), ysp.VideosSearch(name, limit).result()['result']))

def get_video_upload_date(url: str):
    with yt_dlp.YoutubeDL({"quiet": True, "no_warnings": True}) as ydl:
        info =  ydl.extract_info(url, download=False)
        return info.get("upload_date", datetime.today())
